<?php
/********************************* PHP-P10 ******************************
 *    P10 uplink class by pk910   (c)2011 pk910                         *
 ************************************************************************
 *                          Version 2 (OOP)                             *
 *                                                                      *
 * PHP-P10 is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with PHP-P10; if not, write to the Free Software Foundation,   *
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.       *
 *                                                                      *
 ************************************************************************
 * 
 *  BotLoader/BotLoader.class.php
 *
 * This class loades / rehashs or unloads all Bots.
 *
 */
require_once("Bot.class.php");
require_once("saxdb.class.php");

class BotLoader {
	const BOT_DIR = "Bots";
	const TMP_DIR = "tmp";
	private $uplink;
	private $saxdb;
	private $botdb;
	private $loadedBots = array();
	private $botDatabases = array();
	private static $botloader;
	
	public function __construct($uplink) {
		$this->uplink = $uplink;
		$this->saxdb = new saxdb();
		$this->saxdb->loadDB("php_p10.db");
		$this->botdb = $this->saxdb->getSection("BotLoader");
		self::$botloader = $this;
		timer(60*10, array($this, "autosave"), array());
	}
	
	public function loadBots() {
		if(array_key_exists("bots", $this->botdb) && is_array($this->botdb["bots"])) {
			foreach($this->botdb["bots"] as $name => $botfile) {
				$this->loadBot($name, $botfile);
			}
		} else {
			$this->botdb["bots"] = array();
		}
	}
	
	public function unloadBots() {
		foreach($this->loadedBots as $name => $bot) {
			$this->unloadBot($name, false);
		}
	}
	
	public function save() {
		$this->saxdb->setSection("BotLoader", $this->botdb);
		$this->saxdb->writeDB("php_p10.db");
	}
	
	public function autosave() {
		foreach($this->loadedBots as $name => $bot) {
			if(array_key_exists(strtolower($name), $this->botDatabases)) {
				$db = $bot->writeDB();
				$this->saxdb->setSection($this->botDatabases[strtolower($name)],$db);
			}
		}
		$this->save();
		timer(60*10, array($this, "autosave"), array());
	}
	
	public function loop() {
		foreach($this->loadedBots as $name => $bot) {
			$bot->loop();
		}
	}
	
	private function loadBot($name, $botfile) {
		if(array_key_exists(strtolower($name), $this->loadedBots)) return false;
		//load bot
		$bot = $this->loadClass($botfile, $name);
		if(!$bot) return false;
		$bot->load($this->uplink);
		$this->loadedBots[strtolower($name)] = $bot;
		if(!array_key_exists($name, $this->botdb["bots"])) {
			$this->botdb["bots"][$name] = $botfile;
		}
		return true;
	}
	
	private function unloadBot($name, $delete = true) {
		if(!(array_key_exists(strtolower($name), $this->loadedBots))) return false;
		//unload bot
		$bot = $this->loadedBots[strtolower($name)];
		if(array_key_exists(strtolower($name), $this->botDatabases)) {
			$db = $bot->writeDB();
			$this->saxdb->setSection($this->botDatabases[strtolower($name)],$db);
			unset($this->botDatabases[strtolower($name)]);
		}
		ModCMD::unbindBot($bot);
		$bot->unload(false);
		unset($this->loadedBots[strtolower($name)]);
		if(array_key_exists($name, $this->botdb["bots"]) && $delete) {
			unset($this->botdb["bots"][$name]);
		}
		return true;
	}
	
	private function rehashBot($name) {
		if(!(array_key_exists(strtolower($name), $this->loadedBots))) return false;
		$botfile = null;
		foreach($this->botdb["bots"] as $botname => $file) {
			if(strtolower($botname) == strtolower($name)) {
				$botfile = $file;
				break;
			}
		}
		if(!$botfile) break;
		//rehash bot
		$bot = $this->loadedBots[strtolower($name)];
		if(array_key_exists(strtolower($name), $this->botDatabases)) {
			$db = $bot->writeDB();
			$this->saxdb->setSection($this->botDatabases[strtolower($name)],$db);
			unset($this->botDatabases[strtolower($name)]);
		}
		ModCMD::unbindBot($bot);
		$data = $bot->unload(true);
		unset($this->loadedBots[strtolower($name)]);
		//ok  bot is unloaded... load it again...
		$bot = $this->loadClass($botfile, $name);
		if(!$bot) return false;
		$bot->load($this->uplink, $data);
		$this->loadedBots[strtolower($name)] = $bot;
		return true;
	}
	
	private function listLoadedBots() {
		return $this->loadedBots;
	}
	
	private function addDBsection($bot, $section) {
		if(!is_a($bot, "Bot")) return false;
		$name = null;
		foreach($this->loadedBots as $botname => $cbot) {
			if($cbot === $bot) {
				$name = $botname;
				break;
			}
		}
		if($name == null) return false;
		if(array_key_exists(strtolower($name), $this->botDatabases)) {
			return false;
		}
		$this->botDatabases[strtolower($name)] = $section;
		$bot->readDB($this->saxdb->getSection($section));
	}
	
	private function loadClass($file, $classprefix) {
		$dir = self::BOT_DIR;
		$tmp = self::TMP_DIR;
		$maincode=file_get_contents($dir."/".$file);
		if(!$maincode) return;
		$class = rand(10000,99999);
		while(class_exists("bot_".$classprefix."_".$class)) {
			$class = rand(10000,99999);
		}
		$maincode = str_replace('{$_NAME}', "module_".$classprefix."_".$class, $maincode);
		$fp = fopen($tmp."/modules_".$classprefix."_".$class.".tmp.php", 'w');
		fwrite($fp, $maincode);
		fclose($fp);
		include($tmp."/modules_".$classprefix."_".$class.".tmp.php");
		$classname = "module_".$classprefix."_".$class;
		$newclass = new $classname();
		unlink($tmp."/modules_".$classprefix."_".$class.".tmp.php");
		return $newclass;
	}
	
	public static function load($name, $botfile) {
		return self::$botloader->loadBot($name, $botfile);
	}
	
	public static function unload($name) {
		return self::$botloader->unloadBot($name);
	}
	
	public static function rehash($name) {
		return self::$botloader->rehashBot($name);
	}
	
	public static function listBots() {
		return self::$botloader->listLoadedBots();
	}
	
	public static function registerDB($bot, $name) {
		return self::$botloader->addDBsection($bot, $name);
	}
}

?>