<?php
/********************************* PHP-P10 ******************************
 *    P10 uplink class by pk910   (c)2011 pk910                         *
 ************************************************************************
 *                          Version 2 (OOP)                             *
 *                                                                      *
 * PHP-P10 is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with PHP-P10; if not, write to the Free Software Foundation,   *
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.       *
 *                                                                      *
 ************************************************************************
 * 
 *  Uplink/Numerics.class.php
 *
 * P10 numeric functions
 *
 ************************************************************************
 * accessable functions
 *
 * static String intToNum(int $int, int $length)
 *     returns the numeric representing $int
 *
 * static int numToInt(String $numeric)
 *     returns the integer value, the numeric represents
 *
 * static String parseIP(String $numeric)
 *     parses an IP Address in numeric format
 *
 * static String numericFromIP(String $ip)
 *     builds a numeric representing the IP
 */

class Numerics {
	private static $base64chars = array(
	  'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
	  'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
	  'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
	  'w','x','y','z','0','1','2','3','4','5','6','7','8','9','[',']'
	);
	private static $base64charsLength = 64;
	
	public static function intToNum($int, $length) {
		//fix a small "bug": normaly 0 = AAAAA but we need 1 = AAAAA
		$int = $int - 1;
		
		$numeric = "";
		for($pos = $length-1; $pos >= 0; $pos--) {
			//current position represents floor($int / ($base64charsLength ^ $pos))
			$base = 1;
			for($i = 0; $i < $pos; $i++) {
				$base = $base * self::$base64charsLength;
			}
			$posValue = floor($int / $base);
			//get the char representing $posValue
			$posChar = self::$base64chars[$posValue];
			$numeric .= $posChar;
		}
		
		return $numeric;
	}
	
	public static function numToInt($numeric) {
		$base = 1;
		$int = 0;
		for($pos = strlen($numeric)-1; $pos >= 0; $pos--) {
			$posValue = array_search($numeric[$pos], self::$base64chars);
			$int = ($posValue * $base);
			$base = $base * self::$base64charsLength;
		}
		
		//fix a small "bug": normaly 0 = AAAAA but we need 1 = AAAAA
		$int = $int + 1;
		
		return $int;
	}
	
}

?>